@@ -1,6 +1,6 @@
 /* pk-cmd-editor.c - Command to invoke an external editor.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ pk_cmd_editor (int argc, struct pk_cmd_arg argv[], uint64_t uflags)
     return 1;
 
   /* editor */
-  assert (argc == 0);
+  assert (argc == 1);
 
   /* Get the value of EDITOR.  */
   editor = getenv ("EDITOR");
@@ -144,4 +144,4 @@ pk_cmd_editor (int argc, struct pk_cmd_arg argv[], uint64_t uflags)
 }
 
 const struct pk_cmd editor_cmd =
-  {"editor", "", "", 0, NULL, pk_cmd_editor, ".editor", NULL};
+  {"editor", "", "", 0, NULL, NULL, pk_cmd_editor, ".editor", NULL};
